#ifndef Morse_h
#define Morse_h

#include "Arduino.h"
#include <avr/pgmspace.h>
#include <inttypes.h>

// Milliseconds of debounce time.
#define DEBOUNCE_TIME 10

#define KEYTIME_POS_SIZE 32

class Morse {
  public:
    Morse(Print *);
    
    uint8_t poll();
    uint8_t available();
    uint8_t read();

    void SaveState();

    char *getCode(char letter);
  private:
    Print *printer;
    
    void resetMap();
    char LoadState();
    unsigned int _keyTimeMap[KEYTIME_POS_SIZE];
    char _keyTimePos; // Where in the above map we are.
    char _keyLastPos; // Current key position (HIGH or LOW)
    long _keyLastEvent; // time of current key position
    char _caseMode;
    char _useTone;
    char next_space; // indicates inter-word gap

    unsigned int _ditTime;
    unsigned int _dahTime;
    char startLearningMode(char);
    unsigned char getLetterFromCode(const char *);
    unsigned char _lastLearningMode;
    unsigned char _mode;
    void calculateTiming();
    uint8_t getWpm();
    void showMenu();
    unsigned char decodeSymbol();

};
#endif
